

-- LilSparky's Workshop -- Addon for WOW Trade Skill frames
-- Lilsparky of Lothar
-- v1.00
-- apr.29.2009

local LSW_VERSION = "LSW v1.00";
local LSW_VERSION_LONG = "LilSparky's Workshop v1.00"


LSWConfig = {}


AuctionLib = {}

do
	local algorithms = {}
	local algorithmsIndex = {}

	local defaultAlgorithm

	local options = {}

	function AuctionLib:RegisterToggle(name, optionTable, variable)
		local o = {}

		o.name = name
		o.optionTable = optionTable
		o.variable = variable

		if not options then
			options = {}
		end

		table.insert(options, o)
	end


	function AuctionLib:RegisterAlgorithm(name, func)
		if name and func then

			local a = {}
			a.name = name
			a.func = func

			local i = algorithmsIndex[name]

			if i then
				algorithms[i] = a
			else
				table.insert(algorithms, a)

				algorithmsIndex[name] = #algorithms
			end

			if not defaultAlgorithm then
				defaultAlgorithm = a
			end

			return a
		end
	end

	function AuctionLib:GetAlgorithms()
		return algorithms, algorithmsIndex
	end

	function AuctionLib:GetAlgorithmFunction(name)
		if name then
			local index = algorithmsIndex[name]
			if index then
				return algorithms[index].func, algorithms[index].name
			end
		end

		if defaultAlgorithm then
			return defaultAlgorithm.func, defaultAlgorithm.name
		else
			return function () return 0 end, "no auction support"
		end
	end


	function AuctionLib:SetDefaultAlgorithm(algo)
		if algo and algo.fun and algo.name then
			defaultAlgorithm = algo
		end
	end

	function AuctionLib:GetAuctionValue(link, a)
		return algorithms[algorithmsIndex[a]].func(link)
	end


	function AuctionLib:GenerateAlgorithmMenu(checkedName, callback)
		local menu = {}
		local oldButton

		for i = 1, #algorithms do
			local name = algorithms[i].name


			local button = {
				text = name,
				func = function(b)
					if oldButton then
						oldButton.checked = false
					end
					menu[i].checked = true
					oldButton = menu[i]
					callback(b, name)
				end,
				value = i,
				checked = (checkedName == name),
			}

			table.insert(menu, button)

			if checkedName == name then
				oldButton = menu[i]
			end
		end

		return menu
	end

	function AuctionLib:GenerateOptionsMenu(callback)
		local menu = {}

		for i = 1, #options do
			local o = options[i]

			local button = {
				text = o.name,
				func = function(b)
					o.optionTable[o.variable] = not o.optionTable[o.variable]
					menu[i].checked = o.optionTable[o.variable]
					callback(b, o.name)
				end,
				value = i,
				checked = o.optionTable[o.variable],
			}

			table.insert(menu, button)
		end

		return menu
	end
end





LSW = {}

do
	LSW.version = LSW_VERSION

	local frameSupportList = {}
	local pricingSupportList = {}


	local timerList = {}

	local pricingInitialized = false;

	local LSW_MINIMUM_REAGENT_AUCTIONS = 10
	local LSW_MINIMUM_ITEM_AUCTIONS = 1

	local ENCHANT_ID="?"		-- enchants have no item fate

	local itemFateColor={
		["d"]="ff008000",
		["a"]="ff909050",
		["v"]="ff206080",
		["?"]="ff800000",
	}

--	local LSW_itemFateColor[LSW_ENCHANT_ID]="ffffffff";
	local priceDataSync = 1
	local recipeCache={}
	local itemCache={}


	local globalFate = 0
	local globalFateMax = 2
	local valueAsPercent = false 	-- display % for values

	local globalSource = 0

	-- these could certainly be made language specific if needed
	local itemFateList={"a", "v", "d"}

	local fateString={["a"]="Auction", ["v"]="Vendor", ["d"]="Disenchant"}

--	local LSWTooltip = CreateFrame("GameTooltip", "LSWTooltipFrame", UIParent, "GameTooltipTemplate")
--			LSWTooltip:SetScale(0.8)
	local LSWTooltip = GameTooltip


	local BOP_STRING = "|cffff0000-BOP-|r"
	local NO_DE_STRING = "|cffff0000NO DE|r"


	local aucAlgorithm = "Market"

	local aucEngines = {}


	local buttonValueList = {}
	local buttonCostList = {}
	local buttonLevelList = {}

	local LSWMenuFrame = CreateFrame("Frame", "LSWMenuFrame", UIParent, "UIDropDownMenuTemplate")

	local profitMenu = {}

	local valueMenu =
	{
	}

	local costBasisMenu =
	{
		{
			text = "Item Resale Value",
			tooltipText = "Reagent prices are based on resale value of items\n(what one could make by selling the materials on the open market.)",
			func = function ()
				LSWConfig.costBasis = "resale"
				LSW:FlushPriceData()
			end
		},
		{
			text = "Purchase Cost of Item",
			tooltipText = "Reagent prices are based on the purchase price of materials.\nThis method will consider craftable reagents when determining the price.",
			func = function ()
				LSWConfig.costBasis = "purchase"
				LSW:FlushPriceData()
			end
		},
		{
			text = "Skill Up Cost",
			tooltipText = "Reagent prices are based on the cheapest current auction/vendor price of sufficient quantities for a single skill run.\nAlso, any reagents currently owned are considered free (including those craftable with current inventory).\nReagent prices are then factored by an estimated skill up chance.",
			func = function ()
				LSWConfig.costBasis = "skillUp"
				LSW:FlushPriceData()
			end
		},
	}

	local costMenu =
	{
		{
			text = "Cost Basis",
			menuList = costBasisMenu,
			hasArrow = true,
		}
	}




	function LSW:ChatMessage(msg)
		DEFAULT_CHAT_FRAME:AddMessage("|cff80d060"..(msg or "nil"));
	end

	function LSW:ErrorMessage(msg)
		DEFAULT_CHAT_FRAME:AddMessage("|cffff0000"..(msg or "nil"));
	end


	function LSW:FindID(link)
		if not link then return end

		local itemString = string.match(link, "^|c%x+|H(.+)|h%[.+%]")
		local id

		if itemString then
			_, id = string.split(":",itemString)
		end

		return tonumber(id)
	end


	function LSW:QueryServerInfo(link)
		local id = self:FindID(link);

		if (not id) then
			id = self:FindID(link)

			if ( id) then
				id = "e"..id
			end
		end

		if (id and not self.linkCache[id]) then
			self.linkCache[id] = true;


			if (not GetItemInfo(link)) then
				self:ErrorMessage("no local cache info for "..link);
	--			GameTooltip:SetHyperlink(link);
			end
		end
	end


	function LSW:FormatMoney(moneyString,hilight)
		if not moneyString then
			return nil, "?"
		end
		local neg

		local money = tonumber(moneyString)


		if (money < 0) then
--			return "   --"
			neg = true
			money = -money
		end

		local GSC_GOLD = "ff807000"
		local GSC_SILVER = "ff808080"
		local GSC_COPPER = "ff643016"

		if (hilight) then
			GSC_GOLD="ffffd100"
			GSC_SILVER="ffe6e6e6"
			GSC_COPPER="ffc8602c"
		end

		local g, s, c;
		local digits = 0

		g = math.floor(money/10000);
		s = math.fmod(math.floor(money/100),100);
		c = math.fmod(money,100);


		digits = math.floor(math.log10(money)+1)

		if neg then
			if ( digits < 3 ) then
				gsc = string.format("  |c%s%3d|r",  GSC_COPPER, -c);
			elseif ( digits < 5 ) then
				gsc = string.format("|c%s%3d |c%s%02d|r", GSC_SILVER, -s, GSC_COPPER, c)
			elseif ( digits < 7 ) then
				gsc = string.format("|c%s%3d |c%s%02d|r", GSC_GOLD, -g, GSC_SILVER, s)
			else
				gsc = string.format("|c%s%5d|r", GSC_GOLD, -g);
			end

			return gsc
		else
			if ( digits < 3 ) then
				gsc = string.format("   |c%s%2d|r",  GSC_COPPER, c);
			elseif ( digits < 5 ) then
				gsc = string.format("|c%s%2d|r |c%s%02d|r", GSC_SILVER, s, GSC_COPPER, c)
			elseif ( digits < 7 ) then
				gsc = string.format("|c%s%2d|r |c%s%02d|r", GSC_GOLD, g, GSC_SILVER, s)
			else
				gsc = string.format("|c%s%5d|r", GSC_GOLD, g);
			end

			return gsc
		end
	end


	function LSW:GetTradeSkillData(id)
		local skillName, skillType = GetTradeSkillInfo(id)
		local itemLink = GetTradeSkillItemLink(id)
		local recipeLink = GetTradeSkillRecipeLink(id)

		local itemID = LSW:FindID(itemLink)
		local recipeID = LSW:FindID(recipeLink)

		if itemID and recipeID then
			if itemID == recipeID then
				itemID = -itemID					-- neg itemID = recipeID for enhancements
			end
		end

		return skillName, skillType, itemLink, recipeLink, itemID, recipeID
	end


	local function itemBOPCheck(itemID)
		local tooltip = getglobal("LSWParsingTooltip")
		local bop = false

		if tooltip == nil then
			tooltip = CreateFrame("GameTooltip", "LSWParsingTooltip", getglobal("ANCHOR_NONE"), "GameTooltipTemplate")
			tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")
		end

		tooltip:SetHyperlink("item:"..itemID)

		local tiplines = tooltip:NumLines()

		for i=1, tiplines, 1 do
			local lineText = string.lower(getglobal("LSWParsingTooltipTextLeft"..i):GetText() or " ")

			if (string.find(lineText, "binds when picked up")) then
				bop = true
				break
			end
		end

		return bop
	end




	local function InitializePricing()
		globalFateMax = 2

		local algorithms, algorithmsIndex = AuctionLib:GetAlgorithms()
		local costName, valueName

		LSW.auctionCost, costName = AuctionLib:GetAlgorithmFunction(LSWConfig.costAlgorithm)
		LSW.auctionValue, valueName = AuctionLib:GetAlgorithmFunction(LSWConfig.valueAlgorithm)



		local optionsMenu = AuctionLib:GenerateOptionsMenu(function(button, opt)
			CloseDropDownMenus()
			LSW:FlushPriceData()
		end)




		if #optionsMenu>0 then
			local button = { text = "Options", menuList = optionsMenu, hasArrow = true}
			table.insert(costMenu, button)
			table.insert(valueMenu, button)
			table.insert(profitMenu, button)
		end


		local i = #valueMenu + 1
		local j = #profitMenu + 1

		local temp = AuctionLib:GenerateAlgorithmMenu(valueName, function(button, algo)
			valueMenu[i].text = "Value Module: "..algo
			profitMenu[j].text = "Value Module: "..algo

			LSWConfig.valueAlgorithm = algo
			LSW.auctionValue = AuctionLib:GetAlgorithmFunction(algo)
			CloseDropDownMenus()
			LSW:FlushPriceData()
		end)

		table.insert(valueMenu,{ text = "Value Module: "..valueName, menuList = temp, hasArrow = true })
		table.insert(profitMenu, { text = "Value Module: "..valueName, menuList = temp, hasArrow = true })


		local i = #costMenu + 1
		local j = #profitMenu + 1

		local temp = AuctionLib:GenerateAlgorithmMenu(costName, function(button, algo)
			costMenu[i].text = "Cost Module: "..algo
			profitMenu[j].text = "Cost Module: "..algo

			LSWConfig.costAlgorithm = algo
			LSW.auctionCost = AuctionLib:GetAlgorithmFunction(algo)
			CloseDropDownMenus()
			LSW:FlushPriceData()
		end)

		table.insert(costMenu,{ text = "Cost Module: "..costName, menuList = temp, hasArrow = true })
		table.insert(profitMenu, { text = "Cost Module: "..costName, menuList = temp, hasArrow = true })


		if LibStub then
			local lib, minorVersion = LibStub:GetLibrary("LibPeriodicTable-3.1", true)

			LSW.periodicTableLib = lib

			if (LSW.periodicTableLib) then
				LSW:ChatMessage("LilSparky's Workshop using LibPeriodicTable-3.1 ("..minorVersion..") for vendor availablity data");
			end
		end


		if LSW.disenchantValue then
			globalFateMax = 3
		end


		return true
	end


	local function AddToItemCache(itemID)
--LSW:ChatMessage("add item "..tostring(itemID))
		if itemID and not itemCache[itemID] then
			itemCache[itemID] = {}

			if LSW.periodicTableLib and itemID then
				if LSW.periodicTableLib:ItemInSet(itemID,"Tradeskill.Mat.BySource.Vendor") then
					itemCache[itemID].vendorCost = LSW.vendorCost(itemID)
				end
			end

			itemCache[itemID].vendorValue = LSW.vendorValue(itemID) or 0

--			itemCache[itemID].BOP = itemBOPCheck(itemID)
		end
	end


	local function UpdateRecipeCache()
		local numSkills = GetNumTradeSkills()
		local hasNil

		for i=1, numSkills do
			local name, skillType = GetTradeSkillInfo(i)

			if skillType ~= "header" then
				local itemLink = GetTradeSkillItemLink(i)
				local itemID = LSW:FindID(itemLink)

				local recipeLink = GetTradeSkillRecipeLink(i)
				local recipeID = LSW:FindID(recipeLink)

				if itemID == recipeID then
					itemID = LSW_ScrollID[recipeID]	or -(recipeID or 0)			-- for enchants, the item created is a scroll
				end


				if not recipeCache[recipeID] or recipeCache[recipeID].hasNil then
					AddToItemCache(itemID)

--					local _, binds = GetTradeSkillItemStats(i)					-- boo!

--					if binds == "binds on pickup" then
--						itemCache[itemID].BOP = true
--					end

					if itemID then
						if not itemCache[itemID].craftSource then
							itemCache[itemID].craftSource = {}
						end
						local min, max = GetTradeSkillNumMade(i)

						itemCache[itemID].craftSource[recipeID] = (min + max) / 2

						recipeCache[recipeID] = {}

						recipeCache[recipeID].numMade = (min + max) / 2

						recipeCache[recipeID].numReagents = GetTradeSkillNumReagents(i)
						recipeCache[recipeID].reagents = {}

						recipeCache[recipeID].itemID = itemID

						for j=1,recipeCache[recipeID].numReagents do
							local _,_,count = GetTradeSkillReagentInfo(i,j)
							local itemLink = GetTradeSkillReagentItemLink(i,j)
	--LSW:ChatMessage("recipe "..tostring(recipeLink).." reagent "..tostring(itemLink))

							local itemID = LSW:FindID(itemLink)

							if itemID then
								recipeCache[recipeID].reagents[itemID] = count
								AddToItemCache(itemID)
							else
								recipeCache[recipeID].hasNil = true
								hasNil = true
								i=numSkills
								break
							end
						end
					else
						recipeCache[recipeID].hasNil = true
						hasNil = true
						i=numSkills
						break
					end
				end
			end
		end

		if hasNil then
			LSW:CreateTimer("updateData", 1.0, LSW.UpdateData)
		end
	end


	local function UpdateItemPrices()
		for itemID, cache in pairs(itemCache) do
			if cache.sync ~= priceDataSync then
				cache.sync = priceDataSync

				cache.auctionCost, cache.auctionCostCount = LSW.auctionCost(itemID)
				cache.auctionValue, cache.auctionValueCount = LSW.auctionValue(itemID)

				cache.disenchantValue = (LSW.disenchantValue and LSW.disenchantValue(itemID))

				local a,d,v = cache.auctionValue, cache.disenchantValue or -1, cache.vendorValue

				if d > a then
					if d > v then
						cache.bestValue = d
						cache.fate = "d"
					else
						cache.bestValue = v
						cache.fate = "v"
					end
				else
					if a > v then
						cache.bestValue = a
						cache.fate = "a"
					else
						cache.bestValue = v
						cache.fate = "v"
					end
				end

				if cache.vendorCost then
					if cache.vendorCost < cache.auctionCost or cache.auctionCost < 0 then
						cache.bestCost = cache.vendorCost
						cache.source = "v"
					else
						cache.bestCost = cache.auctionCost
						cache.source = "a"
					end
				else
					cache.bestCost = cache.auctionCost
					cache.source = "a"
				end
			end
		end
	end



	local function UpdateSingleItemPrice(itemID)
		local cache = itemCache[itemID]

		if cache then
			if cache.sync ~= priceDataSync then
				cache.sync = priceDataSync

				cache.auctionCost, cache.auctionCostCount = LSW.auctionCost(itemID)
				cache.auctionValue, cache.auctionValueCount = LSW.auctionValue(itemID)

				if not cache.auctionCost then
					cache.auctionCost, cache.auctionCostCount = -1, 0
				end

				if not cache.auctionValue then
					cache.auctionValue, cache.auctionValueCount = -1, 0
				end


				cache.disenchantValue = (LSW.disenchantValue and LSW.disenchantValue(itemID))

				local a,d,v = cache.auctionValue, cache.disenchantValue or -1, cache.vendorValue

				if d > a then
					if d > v then
						cache.bestValue = d
						cache.fate = "d"
					else
						cache.bestValue = v
						cache.fate = "v"
					end
				else
					if a > v then
						cache.bestValue = a
						cache.fate = "a"
					else
						cache.bestValue = v
						cache.fate = "v"
					end
				end

				if cache.vendorCost then
					if cache.vendorCost < cache.auctionCost or cache.auctionCost < 0 then
						cache.bestCost = cache.vendorCost
						cache.source = "v"
					else
						cache.bestCost = cache.auctionCost
						cache.source = "a"
					end
				else
					cache.bestCost = cache.auctionCost
					cache.source = "a"
				end
			end
		end
	end


	local function UpdateSingleRecipePrice(recipeID)
		local cache = recipeCache[recipeID]

		if cache then
			UpdateSingleItemPrice(cache.itemID)
			if cache.sync ~= priceDataSync then
				local buy = 0
				local sell = 0
				local cost = 0

				cache.sync = priceDataSync

	--			for itemID, numNeeded in pairs(cache.reagents) do
	--				buy = buy + numNeeded * math.max(itemCache[itemID].vendorCost or itemCache[itemID].auctionCost, itemCache[itemID].auctionCost)
	--				sell = sell + numNeeded * math.max(itemCache[itemID].vendorValue, itemCache[itemID].auctionValue)
	--			end

				for itemID, numNeeded in pairs(cache.reagents) do
					UpdateSingleItemPrice(itemID)
					cost = cost + numNeeded * itemCache[itemID].bestCost
				end

				cache.cost = cost
			end
		end
	end


	local function UpdateRecipePrices()
		local numSkills = GetNumTradeSkills()
		local hasNil

		for i=1, numSkills do
			local name, skillType = GetTradeSkillInfo(i)

			if skillType ~= "header" then
				local recipeLink = GetTradeSkillRecipeLink(i)
				local recipeID = LSW:FindID(recipeLink)

				UpdateSingleRecipePrice(recipeID)
			end
		end
	end

	function LSW:FlushPriceData()
		priceDataSync = priceDataSync + 1
		LSW:UpdateData()
	end


	local updateInProgress

	function LSW:UpdateData()
		if not updateInProgress then
			updateInProgress = true


			local startTime = GetTime()

			if not LSW.pricingInitialized then
				LSW.pricingInitialized = true
				InitializePricing()
			end


			UpdateRecipeCache()
--			UpdateItemPrices()
			UpdateRecipePrices()


--	LSW:ChatMessage("updated data in "..floor((GetTime()-startTime)*100)/100 .." seconds")
--	LSW:ChatMessage("done updating")

			updateInProgress = nil


--LSW:ChatMessage("updated data in "..GetTime()-startTime.." seconds")


			if LSW.parentFrame:IsVisible() then
				LSW:RefreshWindow()
			end
		end
	end


	function LSW:GetSkillCost(recipeID)
		if recipeCache[recipeID] then
			return recipeCache[recipeID].cost
		end

		return -1
	end


	function LSW:GetBestSkillValue(recipeID, fate)
		if not fate then
			fate = globalFate
		end

		local value, itemFate

		if recipeCache[recipeID] then
			local cache = itemCache[recipeCache[recipeID].itemID]
			local numMade = recipeCache[recipeID].numMade

			if cache then
				if fate == 0 then
					value, itemFate = cache.bestValue, cache.fate
				elseif fate == 1 then
					value, itemFate = cache.auctionValue, itemFateList[fate]
				elseif fate == 2 then
					value, itemFate = cache.vendorValue, itemFateList[fate]
				else
					value, itemFate = cache.disenchantValue, itemFateList[fate]
				end

				if value and value > 0 then
					value = value * numMade
				end

				return value, itemFate
			end
		end

		return -1, "?"
	end


	local function UpdateWindow()
		if LSW.parentFrame:IsVisible() then
			LSW:RefreshWindow()
		end
	end



	function LSW:SkillButtonShow(button)
		local name = button:GetName()
		local itemLevel = "?"
		local x
		local id
		local alt = IsAltKeyDown()

		id = string.match(name, LSW.buttonNamePattern)
		id = tonumber(id)

		local buttonValue, buttonCost, buttonLevel = LSW:CreateDynamicButtons(id, name)

		width = LSW:GetSkillListWidth()

		if LSWConfig.singleColumn then
			width = width + buttonCost:GetWidth()
			buttonValue:SetWidth(45)
		else
			buttonValue:SetWidth(40)
		end


		button:SetWidth(width)

		if (not buttonValue) then return; end

		local skillID = button:GetID()

		local skillName, skillType, itemLink, recipeLink, itemID, recipeID = LSW:GetTradeSkillData(skillID)


		buttonCost:SetID(skillID)
		buttonValue:SetID(skillID)
		buttonLevel:SetID(skillID)

		if (recipeID and skillType ~= "header") then
			local costAmount = LSW:GetSkillCost(recipeID)
			local valueAmount, itemFate = LSW:GetBestSkillValue(recipeID, globalFate)

			local itemFateString = string.format("|c%s%s|r", itemFateColor[itemFate], itemFate)
			local hilight = (costAmount or 0) < (valueAmount or 0)
			local valueText

			if itemFate == "a" and alt then
				local iCache = itemCache[itemID]
				valueText = (iCache and iCache.auctionValueCount) or 0
			elseif itemFate == "a" and itemCache[itemID] and itemCache[itemID].BOP then
				valueText = BOP_STRING
			elseif itemFate == "d" and itemCache[itemID] and not itemCache[itemID].disenchantValue then
				valueText = NO_DE_STRING
			else
				if LSWConfig.valueAsPercent then
					if (costAmount > 0 and valueAmount >= 0) then
						local per = valueAmount / costAmount

						if per < .1 then
							per = math.floor(per*1000)/10
							valueText = string.format("%2.1f%%",per)
						elseif per > 10 then
							per = math.floor(per*10)/10
							valueText = string.format("%2.1fx",per)
						else
							per = math.floor(per*100)
							valueText = per.."%"
						end

						if (hilight) then
							valueText = "|cffd0d0d0"..valueText..itemFateString
						else
							valueText = "|cffd02020"..valueText..itemFateString
						end

					elseif (valueAmount >= 0) then
						valueText = "inf"..itemFateString
					end
				else
					if LSWConfig.singleColumn then
						valueText = (LSW:FormatMoney((valueAmount or 0) - (costAmount or 0),hilight) or "--")..itemFateString
					else
						if valueAmount < 0 then
							valueText = "   --"..itemFateString
						else
							valueText = (LSW:FormatMoney(valueAmount,hilight) or "--")..itemFateString
						end
					end
				end
			end


			buttonValue.text:SetText(valueText)

			if LSWConfig.singleColumn then
				buttonCost:Hide()
			else
				buttonCost.text:SetText(LSW:FormatMoney(costAmount,false).."  ")
				buttonCost:Show()
			end


			if recipeID then
				buttonValue:Show()

				if itemID>0 and LSWConfig.showLevel then
					local _, _, quality, _, itemLevel = GetItemInfo(itemID)

					if itemLevel then
						local r,g,b,hex = GetItemQualityColor(quality)

						if (itemLevel == 0) then
							itemLevel = "1"
						end

						buttonLevel.text:SetText(hex..itemLevel)
						buttonLevel:Show()
					end
				else
					buttonLevel.text:SetText("")
					buttonLevel:Hide()
				end
			else
				buttonValue:Hide()
				buttonLevel:Hide()
			end
		else
			buttonValue:Hide()
			buttonCost:Hide()
			buttonLevel:Hide()
		end
	end




	function LSW:SkillButtonHide(button)
		local name = button:GetName();
		local id = string.match(name, LSW.buttonNamePattern)
		id = tonumber(id)

		local buttonValue, buttonCost, buttonLevel = LSW:CreateDynamicButtons(id, name)

		if (buttonCost) then buttonCost:Hide(); end
		if (buttonValue) then buttonValue:Hide(); end
		if (buttonLevel) then buttonLevel:Hide(); end

	--	LSW_ChatMessage("Hide "..name);
	end



	local function ItemLevelButton_OnEnter(button)
		LSWTooltip:SetOwner(button, "ANCHOR_NONE")
		LSWTooltip:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -CONTAINER_OFFSET_X - 13, CONTAINER_OFFSET_Y)
		LSWTooltip:SetText("Required level to use crafted item.")
		LSWTooltip:Show()
	end

	local function ItemLevelButton_OnLeave()
		LSWTooltip:Hide()
	end



	local function CostButton_OnEnter(button)
		LSWTooltip:SetOwner(LSW.parentFrame, "ANCHOR_NONE")
		LSWTooltip:SetPoint("BOTTOMLEFT", LSW.parentFrame, "BOTTOMRIGHT")

		local total = 0

		local pad = ""

		if LSWConfig.singleColumn then
			pad = "    "
		end

		if true or LSWConfig.priceSanity then
			local skillID = this:GetID()
			local skillName, skillType, itemLink, recipeLink, itemID, recipeID = LSW:GetTradeSkillData(skillID)

			if skillType ~= "header" then
				LSWTooltip:AddLine(recipeLink)


				local rCache = recipeCache[recipeID]

				if rCache then
					local costAmount = LSW:GetSkillCost(recipeID)

					for itemID, numNeeded in pairs(rCache.reagents) do
						local iCache = itemCache[itemID]
						local _, reagentName = GetItemInfo(itemID)

						if iCache.BOP then
							LSWTooltip:AddDoubleLine(pad..reagentName.." x "..numNeeded, LSW_BOP_STRING)
						else
							LSWTooltip:AddDoubleLine(pad..reagentName.." x "..numNeeded, LSW:FormatMoney(iCache.bestCost * numNeeded, true)..iCache.source, r, g, b);

							total = total + iCache.bestCost * numNeeded
						end
					end
				end

				LSWTooltip:AddDoubleLine("Total estimated cost: ", LSW:FormatMoney(total,true).."  ")
			end
		else
			LSWTooltip:SetText("Estimated cost to use skill.")
		end

		if not LSWConfig.singleColumn then
			LSWTooltip:Show()
		end

		return total
	end


	local function CostButton_OnLeave()
		LSWTooltip:Hide();
	end

	local function CostButton_OnClick(frame, button)
		if (frame:GetText() == " ") then return end

		if (button=="RightButton") then
			EasyMenu(costMenu, LSWMenuFrame, frame, 0,0, "MENU", 5)
		end
	end


	local function ValueButton_OnEnter(button)
		if (button:GetText() == " ") then return end

		local pad = ""

		if LSWConfig.singleColumn then
			pad = "    "
		end

		LSWTooltip:SetOwner(LSW.parentFrame, "ANCHOR_NONE")
		LSWTooltip:SetPoint("BOTTOMLEFT", LSW.parentFrame, "BOTTOMRIGHT")

		local skillID = button:GetID()
		local skillName, skillType, itemLink, recipeLink, itemID, recipeID = LSW:GetTradeSkillData(skillID)

		if (skillName and skillType ~= "header") then
			LSWTooltip:ClearLines()

			local rCache = recipeCache[recipeID]

			if rCache then
				local iCache = itemCache[rCache.itemID]

				local auctionValue = LSW:FormatMoney(iCache.auctionValue,(iCache.fate=="a"))
				local vendorValue = LSW:FormatMoney(iCache.vendorValue,(iCache.fate=="v"))
				local disenchantValue = LSW:FormatMoney(iCache.disenchantValue,(iCache.fate=="d"))

				if iCache.BOP then
					auctionValue = LSW_BOP_STRING
				end

				local itemName, itemLink = GetItemInfo(rCache.itemID)

				if not itemName then
					itemLink = GetSpellLink(recipeID)
				end


				if LSWConfig.singleColumn then
					local totalCost = CostButton_OnEnter(button)
					LSWTooltip:AddLine(" ")
	--				LSWTooltip:AddLine("Result:")
					local best = LSW:GetBestSkillValue(recipeID)

					local bestValue = LSW:FormatMoney(best, true)

					if rCache.numMade > 1 then
						LSWTooltip:AddDoubleLine("    "..itemLink.."x"..rCache.numMade, bestValue..iCache.fate)
						ea = "ea"
					else
						LSWTooltip:AddDoubleLine("    "..itemLink, bestValue..iCache.fate)
					end

					LSWTooltip:AddDoubleLine("Total Estimated Profit:", LSW:FormatMoney(best - totalCost, true))
				else
					local ea = ""

					if rCache.numMade > 1 then
						LSWTooltip:AddLine(itemLink.." x "..rCache.numMade)
						ea = "ea"
					else
						LSWTooltip:AddLine(itemLink)
					end

					if iCache.auctionValue > 0 then
						LSWTooltip:AddDoubleLine("|cff909040Auction("..(iCache.auctionValueCount or "?").."): ",auctionValue..ea)
					else
						LSWTooltip:AddDoubleLine("|cff909040Auction: ","??"..ea)
					end

					LSWTooltip:AddDoubleLine("|cff206080Vendor: ",vendorValue..ea)

					if disenchantValue then
						LSWTooltip:AddDoubleLine("|cff008000Disenchant: ",disenchantValue..ea)
					end

					local cycleString

					if (globalFate == 0) then
						cycleString = "[Best Value Shown]"
					else
						cycleString = "["..fateString[itemFateList[globalFate] ].." Value Shown]"
					end

					LSWTooltip:AddLine(cycleString,1,1,1)
				end

				LSWTooltip:Show()
			end
		end
	end



	local function ValueButton_OnClick(frame, button)
		if (frame:GetText() == " ") then return end

		if(button=="LeftButton") then
			globalFate = globalFate + 1;

			if (globalFate > globalFateMax) then
				globalFate = 0;
			end

			ValueButton_OnEnter(frame)

			UpdateWindow()
		elseif (button=="RightButton") then
			if LSWConfig.singleColumn then
				EasyMenu(profitMenu, LSWMenuFrame, frame, 0,0, "MENU", 5)
			else
				EasyMenu(valueMenu, LSWMenuFrame, frame, 0,0, "MENU", 5)
			end
		end
	end


	local function ValueButton_OnLeave()
		LSWTooltip:Hide()
	end




	function LSW:CreateDynamicButtons(id, buttonName)


	--LSW_ErrorMessage("creating button "..id);

		if buttonValueList[id] then
			return buttonValueList[id], buttonCostList[id], buttonLevelList[id]
		end

		local buttonValue = CreateFrame("Button", "LSWTradeSkillValue"..id, self.parentFrame, "LSWTradeSkillValueButtonTemplate")
		buttonValue:SetPoint("TOPLEFT",buttonName,"TOPRIGHT",5,0)

		buttonValue.text = getglobal("LSWTradeSkillValue"..id.."Text")
		buttonValue.text:SetText("00 00")
		buttonValue:Hide()
		buttonValue:SetWidth(40)

		buttonValue:RegisterForClicks("AnyUp")
		buttonValue:SetScript("OnClick", ValueButton_OnClick)
		buttonValue:SetScript("OnEnter", ValueButton_OnEnter)
		buttonValue:SetScript("OnLeave", ValueButton_OnLeave)



		local buttonCost = CreateFrame("Button", "LSWTradeSkillCost"..id, self.parentFrame, "LSWTradeSkillCostButtonTemplate")
		buttonCost:SetPoint("TOPLEFT",buttonName,"TOPRIGHT",buttonValue:GetWidth()+15,0)

		buttonCost.text = getglobal("LSWTradeSkillCost"..id.."Text")
		buttonCost.text:SetText("00 00")

		buttonCost:SetWidth(40)
		buttonCost:Hide()

		buttonCost:RegisterForClicks("AnyUp")
		buttonCost:SetScript("OnClick", CostButton_OnClick)
		buttonCost:SetScript("OnEnter", CostButton_OnEnter)
		buttonCost:SetScript("OnLeave", CostButton_OnLeave)



		local buttonLevel = CreateFrame("Button", "LSWTradeSkillItemLevel"..id, self.parentFrame, "LSWTradeSkillItemLevelTemplate")
		buttonLevel:SetPoint("TOPLEFT",buttonName,"TOPLEFT",5,0)

		buttonLevel.text = getglobal("LSWTradeSkillItemLevel"..id.."Text")
		buttonLevel.text:SetText("00 00")

		buttonLevel:SetText("")
		buttonLevel:Hide()

		buttonLevel:RegisterForClicks("AnyUp")
		buttonLevel:SetScript("OnEnter", ItemLevelButton_OnEnter)
		buttonLevel:SetScript("OnLeave", ItemLevelButton_OnLeave)



		buttonValueList[id] = buttonValue
		buttonCostList[id] = buttonCost
		buttonLevelList[id] = buttonLevel

		return buttonValue, buttonCost, buttonLevel
	end



-- registers support for a ui
-- name is the name of the support (for record keeping essentially)
-- test is the function used to determine if this ui is present
-- init is the function used to initialize
-- frame support modules should register with this function at load time
	function LSW:RegisterFrameSupport(name, test, init)
		local t = {}

		t.name = name
		t.Test = test
		t.Init = init

		table.insert(frameSupportList, t)
	end



-- registers support for an auction module
-- name is the name of the support (for record keeping essentially)
-- test is the function used to determine if this ui is present
-- init is the function used to initialize
-- auction support modules should register with this function at load time
	function LSW:RegisterPricingSupport(name, test, init)
		local t = {}

		t.name = name
		t.Test = test
		t.Init = init

--LSW:ChatMessage("registering pricing module "..name);
		table.insert(pricingSupportList, t)
	end


	function LSW:GetFrameSupportList()
		return frameSupportList
	end


	local function InitializeFrameSupport()
		for i = 1, #frameSupportList do
			if frameSupportList[i].Test(i) then
				LSW:ChatMessage("LSW: Initializing "..frameSupportList[i].name.." frame support")
				frameSupportList[i].Init(i)
				break
			end
		end
	end


	local function InitializePricingSupport()
		for i = 1, #pricingSupportList do
			if pricingSupportList[i].Test(i) then
				LSW:ChatMessage("LSW: Initializing "..pricingSupportList[i].name.." pricing support")
				pricingSupportList[i].Init(i)
			end
		end
	end


	local function OnLoad()
		if not LSW.initialized then
			LSW.initialized = true

			LSW:ChatMessage(LSW_VERSION_LONG)


			AuctionLib:RegisterToggle("Show Value as Percent", LSWConfig, "valueAsPercent")
			AuctionLib:RegisterToggle("Single Column", LSWConfig, "singleColumn")

			InitializeFrameSupport()
			InitializePricingSupport()
		end
	end





	LSW.vendorCost = function() return 0 end
	LSW.vendorValue = function() return 0 end
	LSW.auctionCost = function() return 0 end
	LSW.auctionValue = function() return 0 end

	function LSW:CreateTimer(name, countDown, triggerFunction, repeatTime)
		timerList[name] = {countDown=countDown, triggerFunction=triggerFunction, repeatTime=repeatTime}
	end


	function LSW:DeleteTimer(name)
		timerList[name] = nil
	end

	local master = CreateFrame("Frame", "LSWMasterFrame")

	master:RegisterEvent("ADDON_LOADED")
	master:RegisterEvent("TRADE_SKILL_UPDATE")
	master:RegisterEvent("MODIFIER_STATE_CHANGED")

	master:SetScript("OnEvent", function(frame, event, arg1, arg2)
		if event == "ADDON_LOADED" and arg1 == "LilSparkysWorkshop" then
			master:UnregisterEvent("ADDON_LOADED")
			OnLoad()
		end
		if event == "TRADE_SKILL_UPDATE" then
			if GetTradeSkillLine() then
				LSW:CreateTimer("updateData", 0.1, LSW.UpdateData)
			end
		end

		if event == "MODIFIER_STATE_CHANGED" then
			UpdateWindow()
		end
	end)


	master:SetScript("OnUpdate", function (frame, elapsed)
		if timerList then
			for name,timer in pairs(timerList) do
				timer.countDown = timer.countDown - elapsed
				if timer.countDown <= 0 then
					timer:triggerFunction(timer)

					if timer.repeatTime then
						timer.countDown = timer.countDown + timer.repeatTime
					else
						timerList[name] = nil
					end
				end
			end
		end
	end)
end



